/*
 * sgp4x.h
 *
 *  Created on: Jan 9, 2022
 *      Author: david
 */

#ifndef INC_SGP4X_H_
#define INC_SGP4X_H_


#include "stdint.h"
#include "stm32l0xx_ll_i2c.h"
#include "stm32l0xx_ll_utils.h"
#include "i2c.h"
#include "crc8.h"

/*
 * Defines & macros
 */

#define SGP4X_I2C_ADDRESS 0x59
#define SGP4X_MAX_MEAS_DURATION_MS 50

/*
 * Return values
 */

#define SGP4X_OK 0
#define SGP4X_ERROR -1 // generic error
#define SGP4X_CRC8_ERROR -2 // checksum failed

typedef enum {
	SGP4X_MEASURE_RAW_SIGNAL	= 0x260F,
	SGP4X_EXECUTE_SELF_TEST		= 0x280E,
	SGP4X_TURN_HEATER_OFF		= 0x3615,
	SGP4X_GET_SERIAL_NUMBER		= 0x3682,
	SGP4X_SOFT_RESET			= 0x0006
} sgp4x_cmd_t;

int8_t sgp4x_send_cmd(sgp4x_cmd_t cmd);

int8_t sgp4x_measure_raw_signal (uint16_t * voc_ticks);
int8_t sgp4x_measure_raw_signal_compensated (uint16_t * voc_ticks, uint16_t relative_humidity, int16_t temperature);
int8_t sgp4x_execute_self_test ( uint8_t * test_result);
int8_t sgp4x_get_serial_number ( uint8_t serial[6]);
int8_t sgp4x_turn_heater_off ( void );
int8_t sgp4x_soft_reset ( void );

#endif /* INC_SGP4X_H_ */
